package server.ai;

import common.ai.IEntityMobNPCNode;
import common.entity.Entity;
import common.entity.attributes.AttributeInstance;
import common.entity.attributes.Attributes;
import common.entity.npc.AIHurtByAggressor;
import common.entity.npc.EntityMobNPC;
import common.entity.npc.EntityNPC;
import common.entity.types.EntityLiving;
import common.nbt.NBTTagCompound;

public class EntityMobNPCNode<T extends EntityMobNPC> extends EntityNPCNode<T> implements IEntityMobNPCNode {
    private int angerLevel;
    private int angerTarget;
    
	public EntityMobNPCNode(T entity) {
		super(entity);
	}
	
	protected void registerTasks() {
		super.registerTasks();
        this.targets.addTask(1, new AIHurtByAggressor(this.entity));
	}

    protected void updateAITasks()
    {
        AttributeInstance iattributeinstance = this.entity.getEntityAttribute(Attributes.MOVEMENT_SPEED);

        if (this.isAngry())
        {
            if (!iattributeinstance.hasModifier(Attributes.RUSHING_SPEED_MOD))
            {
                iattributeinstance.applyModifier(Attributes.RUSHING_SPEED_MOD);
            }

            --this.angerLevel;
        }
        else if (iattributeinstance.hasModifier(Attributes.RUSHING_SPEED_MOD))
        {
            iattributeinstance.removeModifier(Attributes.RUSHING_SPEED_MOD);
        }

        if (this.angerLevel > 0 && this.angerTarget != 0 && this.entity.getAttackedBy() == null)
        {
            Entity entity = this.world.getEntityByID(this.angerTarget);
            if(entity instanceof EntityLiving)
            	this.entity.setAttackedBy((EntityLiving)entity);
            if(entity != null && entity.isPlayer())
            	this.entity.playerAttacker = (EntityNPC)entity;
            this.entity.recentlyHit = this.entity.getAttackedTime();
        }

        super.updateAITasks();
    }

    public void writeNbt(NBTTagCompound tagCompound)
    {
        super.writeNbt(tagCompound);
        tagCompound.setShort("Anger", (short)this.angerLevel);
    }

    public void readNbt(NBTTagCompound tagCompund)
    {
        super.readNbt(tagCompund);
        this.angerLevel = tagCompund.getShort("Anger");
    }

    public void becomeAngryAt(Entity entity)
    {
        this.angerLevel = this.rand.excl(400, 800);
        if(entity instanceof EntityLiving)
            this.entity.setAttackedBy((EntityLiving)entity);
    }

    public boolean isAngry()
    {
        return this.angerLevel > 0;
    }
    
    public void setAngerTarget(EntityLiving entity) {
    	this.angerTarget = entity.getId();
    }
}
