package server.init;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Map;
import java.util.Set;

import common.ai.IEntityNode;
import common.collect.Maps;
import common.entity.Entity;
import common.entity.animal.EntityBat;
import common.entity.animal.EntityChicken;
import common.entity.animal.EntityCow;
import common.entity.animal.EntityDragon;
import common.entity.animal.EntityHorse;
import common.entity.animal.EntityMooshroom;
import common.entity.animal.EntityMouse;
import common.entity.animal.EntityOcelot;
import common.entity.animal.EntityPig;
import common.entity.animal.EntityRabbit;
import common.entity.animal.EntitySheep;
import common.entity.animal.EntitySquid;
import common.entity.animal.EntityWolf;
import common.entity.item.EntityBoat;
import common.entity.item.EntityChestCart;
import common.entity.item.EntityCrystal;
import common.entity.item.EntityExplosion;
import common.entity.item.EntityFalling;
import common.entity.item.EntityFireworks;
import common.entity.item.EntityHopperCart;
import common.entity.item.EntityItem;
import common.entity.item.EntityLeashKnot;
import common.entity.item.EntityMinecart;
import common.entity.item.EntityNuke;
import common.entity.item.EntityOrb;
import common.entity.item.EntityTnt;
import common.entity.item.EntityTntCart;
import common.entity.item.EntityXp;
import common.entity.item.EntityXpBottle;
import common.entity.npc.SpeciesInfo;
import common.entity.projectile.EntityArrow;
import common.entity.projectile.EntityBox;
import common.entity.projectile.EntityBullet;
import common.entity.projectile.EntityDie;
import common.entity.projectile.EntityDynamite;
import common.entity.projectile.EntityEgg;
import common.entity.projectile.EntityFireCharge;
import common.entity.projectile.EntityFireball;
import common.entity.projectile.EntityHook;
import common.entity.projectile.EntityPotion;
import common.entity.projectile.EntitySnowball;
import common.entity.types.EntityLiving;
import common.entity.types.IObjectData;
import common.log.Log;
import common.nbt.NBTTagCompound;
import common.world.World;

public abstract class NodeRegistry {
	private static final Map<Class<? extends Entity>, Class<? extends IEntityNode>> NODES = Maps.newHashMap();
	
	private static void registerNode(Class<? extends Entity> clazz, Class<? extends IEntityNode> node) {
		if(clazz == null)
			throw new IllegalArgumentException("Kann keine null-Klasse registrieren");
		if(NODES.containsKey(clazz))
			throw new IllegalArgumentException("Klasse ist bereits registriert: " + clazz);
		NODES.put(clazz, node);
	}
	
	public static void addNode(Entity entity) {
		IEntityNode node = null;
		Class<? extends IEntityNode> oclass = NODES.get(entity.getClass());
		Constructor[] consts = oclass.getConstructors();
		if(consts.length != 1)
			throw new IllegalArgumentException("Klasse " + oclass + " hat mehr als einen Konstruktor");
		try {
			node = (IEntityNode)consts[0].newInstance(entity);
		}
		catch(InstantiationException | IllegalAccessException | InvocationTargetException e) {
			throw new RuntimeException(e);
		}
		entity.setNode(node);
	}
	
	public static Class<? extends IEntityNode> getNodeClass(Class<? extends Entity> clazz) {
		return NODES.get(clazz);
	}
	
	public static void register() {
		registerNode(EntityItem.class, );
		registerNode(EntityXp.class, );
		registerNode(EntityEgg.class, );
		registerNode(EntityLeashKnot.class, );
		registerNode(EntityArrow.class, );
		registerNode(EntitySnowball.class, );
		registerNode(EntityFireball.class, );
		registerNode(EntityFireCharge.class, );
		registerNode(EntityOrb.class, );
		registerNode(EntityPotion.class, );
		registerNode(EntityXpBottle.class, );
		registerNode(EntityBox.class, );
		registerNode(EntityTnt.class, );
		registerNode(EntityFalling.class, );
		registerNode(EntityFireworks.class, );
		registerNode(EntityBoat.class, );
		registerNode(EntityMinecart.class, );
		registerNode(EntityChestCart.class, );
		registerNode(EntityTntCart.class, );
		registerNode(EntityHopperCart.class, );
		registerNode(EntityHook.class, );
		registerNode(EntityDynamite.class, );
		registerNode(EntityNuke.class, );
		registerNode(EntityDie.class, );
		registerNode(EntityExplosion.class, );
		registerNode(EntityCrystal.class, );
		registerNode(EntityBullet.class, );

		registerNode(EntityDragon.class, );
		registerNode(EntityBat.class, );
		registerNode(EntityPig.class, );
		registerNode(EntitySheep.class, );
		registerNode(EntityCow.class, );
		registerNode(EntityChicken.class, );
		registerNode(EntitySquid.class, );
		registerNode(EntityWolf.class, );
		registerNode(EntityMooshroom.class, );
		registerNode(EntityOcelot.class, );
		registerNode(EntityHorse.class, );
		registerNode(EntityRabbit.class, );
		registerNode(EntityMouse.class, );

		for(int z = 0; z < SpeciesRegistry.SPECIMEN.size(); z++) {
			SpeciesInfo info = SpeciesRegistry.SPECIMEN.get(z);
			registerNode(info.clazz, );
        }
	}
}
